// IoT Brick Library
// 1.00 - 2017-05-17
// 1.01 - 2017-06-15
// 1.02 - 2017-06-23
// (c) Copyright 2017
//
// Zur Verwendung mit dem Allnet IoT-Brick


#include <MCP3421.h>
#include <ESP8266WiFi.h>
#include <SSD1306Wire.h>
#include <TimeLib.h>
#include <NtpClientLib.h>
#include <WiFiManager.h>
#include <EEPROM.h>
#include "ESP8266WebServer.h"

#include "all_Type.h"
#include "all_String.h"
#include "all_Terminal.h"


// *** Globale Konstanten ***

const bool t_ShowMessages               = true;      // Status-Mitteilungen anzeigen
const bool t_HideMessages               = false;     // Status-Mitteilungen nicht anzeigen
const int  IoT_Analog10bit              = 0;         // ADC im ESP8266 hat den Index 0


// *** Globale Variablen ***

MCP3421          IoT_ADC18bit           = MCP3421(); // Interner 18 Bit Wandler vom Typ MCP3421
SSD1306Wire      IoT_Display(0x3c, 4, 5);            // Variable für das OLED Display, wird hier initialisiert
ESP8266WebServer IoT_WebServer(80);                  // Webserver starten auf Port 80

bool             IoT_WLANinited         = false;     // Variable für den erfolgreichen (oder nicht) WLAN Connect
bool             IoT_OLEDinited         = false;     // Variable für die Initialisierung der OLED-Display Benutzung
bool             IoT_NTPinited          = false;     // Variable für die Initialisierung der NTP Benutzung
bool             IoT_NTPeventAvail      = false;     // Variable für das Auftreten eines NTP Events
NTPSyncEvent_t   IoT_NTPcurrentEvent;                // Der zuletzt vom TimeServer empfangene Event
long             IoT_EEPROMsize         = 0;         // Größe des verbauten EPROMs


// *** Variablen für verwendete DHCP-IP-Adressen ***

byte             IoT_dynamicIP_address0 = 0;         // 4 Bytes IP-Adresse
byte             IoT_dynamicIP_address1 = 0;
byte             IoT_dynamicIP_address2 = 0;
byte             IoT_dynamicIP_address3 = 0;
byte             IoT_dynamicIP_gateway0 = 0;         // 4 Bytes Gateway
byte             IoT_dynamicIP_gateway1 = 0;
byte             IoT_dynamicIP_gateway2 = 0;
byte             IoT_dynamicIP_gateway3 = 0;
byte             IoT_dynamicIP_subnet0  = 0;         // 4 Bytes Subnetzmaske
byte             IoT_dynamicIP_subnet1  = 0;
byte             IoT_dynamicIP_subnet2  = 0;
byte             IoT_dynamicIP_subnet3  = 0;


// *** Forward-Deklarationen ***

void IoT_Idle ();
void IoT_WatchDog (bool active);
void IoT_DisplayClear (int fontSize);
void IoT_DisplayFontSize (int fontSize);
void IoT_DisplayUpdate ();
void IoT_DisplayDrawText (int x, int y, String text);
void IoT_DisplayAlignText (OLEDDISPLAY_TEXT_ALIGNMENT alignment);


// *** Initialisierung des IoT-Bricks ***

void IoT_Init (bool IoTbrick)                       // Standard-Initialisierung, muss als Erstes aufgerufen werden
{
  IoT_WatchDog(true);                               // IoT-Brick-Watchdog auf 5 Sekunden festlegen
  pinMode(0, INPUT);                                // Eingebauten Taster benutzbar machen
  switch (t_CPUtype())
  {
    case t_ESP8266:
      Serial.begin(115200);
      IoT_EEPROMsize = 4096;
      EEPROM.begin(IoT_EEPROMsize);                 // Daten aus dem EEPROM auslesen
      break;
    case t_ArduinoMega2560:
      Serial.begin(921600);
      break;
    case t_Teensy31:
      Serial.begin(921600);                         // Beherrscht alle Baudraten, macht bis zu 14 MBit.
      break;
    default: // t_ArduinoNano, t_ArduinoDue u.a.
      Serial.begin(115200);
      break;
  }

  if (IoTbrick)
  {
    IoT_Display.init();                             // Initialisiert die OLED graphische Anzeige
    IoT_OLEDinited = true;                          // Variable für OLED-Initialisierung setzen
    //IoT_Display.flipScreenVertically();           // Anzeige spiegeln (optional, hier nur zur Veranschaulichung)
    IoT_DisplayClear(10);                           // Eingebautes OLED-Display löschen
    pinMode(13, OUTPUT);                            // GPIO 13 aus Ausgabe schalten
    pinMode(14, OUTPUT);                            // GPIO 14 aus Ausgabe schalten
    digitalWrite(13, LOW);                          // GPIO 13 auf 0V schalten
    digitalWrite(14, LOW);                          // GPIO 14 auf 0V schalten
    IoT_ADC18bit.init(0x68,3,0);                    // Init MCP3421: I2C-Adresse, 18 Bit Modus, keine Verstärkung
  }
}


// *** WLAN des IoT-Bricks ***

String IoT_WLANstatus ()
{
  switch (WiFi.status())
  {
    case 0:
      return ("Idle");
      break;
    case 1:
      return ("No SSID available");
      break;
    case 2:
      return ("Scan completed");
      break;
    case 3: // WL_CONNECTED
      return ("Connected");
      break;
    case 4:
      return ("Connect failed");
      break;
    case 5:
      return ("Connection lost");
      break;
    case 6:
      return ("Disconnected");
      break;
    default:
      return ("N/A");
      break;
  }
  IoT_Idle();
}

bool IoT_WLANconnected ()                                       // Ist eine WLAN-Verbindung erfolgreich hergestellt?
{
  return (WiFi.isConnected());
}

bool IoT_WLANconnect (String WLANssid, String WLANpassword)     // Funktion um die WLAN Verbindung aufzubauen
{                                                               // incl. Fortschrittsbalken
  IoT_WLANinited = false;
  int progress = 0;

  Serial.print(chr(12));

  // Da manche Access Points langsamer antworten als andere, muss man ein paar Sekunden auf die Antwort warten.
  // Solange WLAN nicht verbunden ist, Fortschrittsbalken im Display und Punkte in der Console anzeigen.
  while (not(IoT_WLANinited) && (progress < 100))
  {  
    if ((progress % 25) == 0)
    {
      if (progress == 50)                                       // Wenn nach 50 versuchen noch keine Wifi-Verbindung
      {                                                         // zu Stande kommt, dann alle Konfigurationsdaten
        Serial.println("");                                     // aus dem Flash-Speicher löschen
        ESP.eraseConfig(); 
        Serial.print("Resetting Configuration. ");
      }
      if ((progress == 0) || (progress == 50))
      {
        Serial.println("WLAN access point " + WLANssid);
        Serial.print("Connecting ");
      }
      IoT_Idle();
      WiFi.persistent(false);                                   // Räumt mit alten RF-Daten auf und sorgt dafür, dass
      WiFi.mode(WIFI_OFF);                                      // zuverlässig eine WiFi-Verbindung aufgebaut wird,
      WiFi.mode(WIFI_STA);                                      // auch dann, wenn WiFi erst später startet. WiFi erst
      WiFi.begin(WLANssid.c_str(), WLANpassword.c_str());       // ausschalten und dann wieder einschalten.
      delay(500);                                               // 500 ms. warten
    }
    IoT_WLANinited = (IoT_WLANstatus() == "Connected");
    //Serial.println(String(progress) + " Status = " + IoT_WLANstatus() + " = " + boolstr(IoT_WLANinited, "True", "False"));
    Serial.print(".");
    
    if (IoT_OLEDinited)
    {
      IoT_Display.clear();
      IoT_Display.drawProgressBar(0, 32, 120, 10, progress);
      IoT_Display.setTextAlignment(TEXT_ALIGN_CENTER);
      IoT_Display.drawString(64, 15, String(progress) + "%");
      IoT_Display.display();
    }
    IoT_Idle();

    if (not(IoT_WLANinited))                // Verbindung noch nicht hergestellt?
    {
      delay(500);                           // 500 ms. warten
      progress++;                           // Durchgang um 1 erhöhen
    }
  }
  Serial.println("");
  if (IoT_WLANinited) // Falls Verbindung erfolgreich, zeige WLAN Namen und die DHCP IP Adresse in der Console
  {
    Serial.print("Connected to ");
    Serial.print(WiFi.SSID());
    Serial.print(", IP: ");
    Serial.println(WiFi.localIP());
    IoT_dynamicIP_address0 = WiFi.localIP()[0];                 // 4 Bytes IP-Adresse
    IoT_dynamicIP_address1 = WiFi.localIP()[1];
    IoT_dynamicIP_address2 = WiFi.localIP()[2];
    IoT_dynamicIP_address3 = WiFi.localIP()[3];
    IoT_dynamicIP_gateway0 = WiFi.gatewayIP()[0];               // 4 Bytes Gateway
    IoT_dynamicIP_gateway1 = WiFi.gatewayIP()[1];
    IoT_dynamicIP_gateway2 = WiFi.gatewayIP()[2];
    IoT_dynamicIP_gateway3 = WiFi.gatewayIP()[3];
    IoT_dynamicIP_subnet0  = WiFi.subnetMask()[0];              // 4 Bytes Subnetzmaske
    IoT_dynamicIP_subnet1  = WiFi.subnetMask()[1];
    IoT_dynamicIP_subnet2  = WiFi.subnetMask()[2];
    IoT_dynamicIP_subnet3  = WiFi.subnetMask()[3];
  }
  else // gebe Fehlermeldung in der Console aus, falls Verbindung fehlgeschlagen
  {
    Serial.println("Connection failed");
  }
  
  return (IoT_WLANinited);
}

String IoT_WLANaddress (bool leadingZero) // Lokale IP-Adresse des WLAN-Netzwerkes (wahlweise mit führenden Nullen)
{
  return (strIP(WiFi.localIP(), leadingZero));
}

String IoT_WLANgateway (bool leadingZero) // Gateway (Router) IP-Adresse des WLAN-Netzwerkes (wahlweise mit führenden Nullen)
{
  return (strIP(WiFi.gatewayIP(), leadingZero));
}

String IoT_WLANsubnet (bool leadingZero) // Subnetzmaske des WLAN-Netzwerkes (wahlweise mit führenden Nullen)
{
  return (strIP(WiFi.subnetMask(), leadingZero));
}

String IoT_WLANssid () // Name des WLAN-Netzwerkes
{
  return (WiFi.SSID());
}

int IoT_WLANrssi () // Signalstärke in dBm (immer ein negativer Wert, z.B. -70 dBm)
{
  return (WiFi.RSSI());
}

String IoT_WLANuptime (bool germanLanguage)                         // Dauer der aktuellen WLAN-Verbindung
{
  String d = NTP.getUptimeString();                                 // Zeitdauer im Format "T days hh:mm:ss"
  if (germanLanguage)
  {
    return (replace(d, "days", "Tage"));                            // Zeitdauer im Format "T Tage hh:mm:ss"
  }
  else
  {
    return (d);                                                     // Zeitdauer im Format "T days hh:mm:ss"
  }
}

String IoT_WLANstartDatetime ()                                     // Startzeit der aktuellen WLAN-Verbindung
{
  String d = NTP.getTimeDateString(NTP.getFirstSync()).c_str();     // Datum & Uhrzeit im Format tt/mm/jjjj hh:mm:ss
  return (replace(d, "/", "."));                                    // Datum & Uhrzeit im Format tt.mm.jjjj hh:mm:ss
}

void IoT_WLANautoConnect (bool useSavedSettings)                    // Verbindung mit WiFi-Manager herstellen
{
  WiFiManager wifiManager;
  if (not(useSavedSettings))
  {
    wifiManager.resetSettings();
  }
  if (IoT_OLEDinited)
  {
    IoT_DisplayClear(16);                                           // OLED Display löschen (16 Punkt Schrift)
    IoT_DisplayAlignText(TEXT_ALIGN_CENTER);
    IoT_DisplayDrawText(63, 0, "WLAN wählen:");
    IoT_DisplayFontSize(24);
    IoT_DisplayDrawText(63, 20, "IoT-Brick");
    IoT_DisplayFontSize(10);
    IoT_DisplayDrawText(63, 50, "Danach HotSpot wählen");
    IoT_DisplayUpdate();                                            // Display aktualisieren
    IoT_DisplayClear(16);                                           // OLED Display löschen (16 Punkt Schrift)
  }
  wifiManager.autoConnect("IoT-Brick");                             // Verbindung ggf. mit gespeicherten Einstellungen 
  if (IoT_OLEDinited)
  {
    IoT_DisplayUpdate();                                            // Display aktualisieren
  }
  Serial.print("Connected to ");
  Serial.print(WiFi.SSID());
  Serial.print(", IP: ");
  Serial.println(WiFi.localIP());
  IoT_WLANinited = true;
  IoT_dynamicIP_address0 = WiFi.localIP()[0];                       // 4 Bytes IP-Adresse
  IoT_dynamicIP_address1 = WiFi.localIP()[1];
  IoT_dynamicIP_address2 = WiFi.localIP()[2];
  IoT_dynamicIP_address3 = WiFi.localIP()[3];
  IoT_dynamicIP_gateway0 = WiFi.gatewayIP()[0];                     // 4 Bytes Gateway
  IoT_dynamicIP_gateway1 = WiFi.gatewayIP()[1];
  IoT_dynamicIP_gateway2 = WiFi.gatewayIP()[2];
  IoT_dynamicIP_gateway3 = WiFi.gatewayIP()[3];
  IoT_dynamicIP_subnet0  = WiFi.subnetMask()[0];                    // 4 Bytes Subnetzmaske
  IoT_dynamicIP_subnet1  = WiFi.subnetMask()[1];
  IoT_dynamicIP_subnet2  = WiFi.subnetMask()[2];
  IoT_dynamicIP_subnet3  = WiFi.subnetMask()[3];
}


// *** Web-Server (HTTP) des IoT-Bricks ***

void IoT_WebUpdate (String *html)
{
  IoT_Idle();
  IoT_WebServer.handleClient();                                     // Bediene die http Anfragen
  IoT_WebServer.send(200, "text/html", *html);                      // HTTP-Statuscode 200 = OK (Anfrage erfolgreich)
  IoT_WebServer.handleClient();                                     // Bediene die http Anfragen
  IoT_Idle();
}

void IoT_WebPrintAllFields ()
{
  String s = "";
  byte argumentCount = IoT_WebServer.args();                        // Anzahl der Felder auf der Web-Seite
  if (argumentCount > 0)                                            // Ist überhaupt eine Eingabe vorhanden?
  {
    for (byte i = 0; i < argumentCount; i++)                        // Alle erlaubten Eingabefelder durchsuchen
    {
      Serial.print("Field #" + strform(i, 48, 2, true) + ": ");     // Die Ausgabe von Unicodes stört die Terminal-Ausgabe,
      s = cleanasc(IoT_WebServer.argName(i));                       // daher hier alle Unicode-Zeichen sinnvoll ersetzen
      Serial.print(left(s + spc(16), 16));                          // Feldname ausgeben, max. 16 Zeichen (Asc II)
      Serial.print(" = " + chr(34));                                // Feldinhalt in Anführungszeichen ausgeben,
      Serial.print(cleanasc(IoT_WebServer.arg(i)));                 // Alle Unicode-Zeichen sinnvoll ersetzen                                      
      Serial.println(chr(34));                                      // max. 32 Zeichen (Asc II)
    }
  }
  IoT_Idle();
}

bool IoT_WebTestField (String varName)                              // Ergibt true, wenn ein Web-Feld mit dem Namen existiert
{
  byte argumentCount = IoT_WebServer.args();                        // Anzahl der Felder auf der Web-Seite
  if (argumentCount > 0)                                            // Ist überhaupt eine Eingabe vorhanden?
  {
    for (byte i = 0; i < argumentCount; i++)                        // Alle erlaubten Eingabefelder durchsuchen
    {
      if (IoT_WebServer.argName(i) == varName)                      // Das Feld mit dem angegebenen varName suchen
      {
         return (true);                                             // Feld auslesen und als Funktionsergebnis übergeben
      }
     }
  }
  return (false);
}

String IoT_WebGetField (String varName)                             // Ergibt den Inhalt des Web-Feldes
{
  byte argumentCount = IoT_WebServer.args();                        // Anzahl der Felder auf der Web-Seite
  if (argumentCount > 0)                                            // Ist überhaupt eine Eingabe vorhanden?
  {
    for (byte i = 0; i < argumentCount; i++)                        // Alle erlaubten Eingabefelder durchsuchen
    {
      if (IoT_WebServer.argName(i) == varName)                      // Das Feld mit dem angegebenen varName suchen
      {
         return (IoT_WebServer.arg(i));                             // Feld auslesen und als Funktionsergebnis übergeben
      }
    }
  }
  return ("");
}


// *** Web-Seitenaufbau ***

String IoT_WebFormOpen (String formName)
{
  String s = "<form action = 'http://" + IoT_WLANaddress(false) + "/" + formName + "' method='POST'>";
  return (s);
}

String IoT_WebFormClose ()
{
  String s = "</form>";
  return (s);
}

String IoT_WebFormSubmitButton (String formName)
{
  String s = "<input type='submit' value='" + cleanhtml(formName) + "'>";
  return (s);
}

String IoT_WebFormActionButton (String actionName, String title)
{
  String s = IoT_WebFormOpen(actionName) + IoT_WebFormSubmitButton(title) + IoT_WebFormClose();
  return (s);
}

String IoT_WebCheckBox (String title, String varName, String varValue, bool checked)
{
  String s = "<input type='checkbox' name='" + varName + "' value='" + varValue + "'";
  if (checked)
  {
    s += " checked";
  }
  s += ">";
  s += cleanhtml(title);
  return (s);
}

String IoT_WebRadioButton (String title, int varIndex, String varName, String varValue, int selectedIndex)
{
  String s = "<input type='radio' id='" + varName + strform(varIndex, 48, 3, false) + "' name='" + varName + "' value='" + varValue + "'";
  if (varIndex == selectedIndex)
  {
    s += " checked";
  }
  s += ">";
  s += cleanhtml(title);
  return (s);
}

String IoT_WebInput (String title, String varName, String varValue, int width)
{
  String s = cleanhtml(title) + "<input type='text' name='" + varName + "' value='" + varValue + "' size='" + str(width) + "'>";
  return (s);
}

String IoT_WebClientIP ()
{
  String s = "<script type='text/javascript' src='https://l2.io/ip.js'></script>";
  return (s);
}

String IoT_WebHtab (int h)
{
  String s = "<span style='padding-left:" + String(h) + "px;'></span>";
  return (s);
}


// *** OLED-Display Funktionen ***

void IoT_DisplayFontSize (int fontSize)           // Schriftgröße setzen
{
  if (IoT_OLEDinited)
  {
    switch (fontSize)
    {
      case 10:
        IoT_Display.setFont(ArialMT_Plain_10);      // Schriftart setzen. Hiermit sind 6 Zeilen Text möglich
        break;
      case 16:
        IoT_Display.setFont(ArialMT_Plain_16);      // Schriftart setzen. Hiermit sind 4 Zeilen Text möglich
        break;
      case 24:
        IoT_Display.setFont(ArialMT_Plain_24);      // Schriftart setzen. Hiermit sind 2 Zeilen Text möglich
        break;
      default:
        IoT_Display.setFont(ArialMT_Plain_10);      // Schriftart setzen. Hiermit sind 6 Zeilen Text möglich
        break;
    }
  }
  IoT_Idle();
}

void IoT_DisplayClear (int fontSize)              // Bildschirm löschen und Schriftgröße setzen
{
  if (IoT_OLEDinited)
  {
    IoT_Display.clear();
    IoT_Display.setTextAlignment(TEXT_ALIGN_LEFT);  // Linksbündige Darstellung des Textes
    IoT_DisplayFontSize(fontSize);
  }
}

void IoT_DisplayUpdate ()                         // Bildschirm aktualisieren
{
  if (IoT_OLEDinited)
  {
    IoT_Display.display();
  }
  IoT_Idle();
}

void IoT_DisplayDrawText (int x, int y, String text) // Text an der angegebenen Koordinate schreiben
{
  if (IoT_OLEDinited)
  {
    IoT_Display.drawString(x, y, text);
  }
  IoT_Idle();
}

void IoT_DisplayAlignText (OLEDDISPLAY_TEXT_ALIGNMENT alignment)
{
  if (IoT_OLEDinited)
  {
    IoT_Display.setTextAlignment(alignment);
  }
}

void IoT_DisplayWLANstatus ()
{
  if (IoT_OLEDinited)
  {
    IoT_DisplayClear(10);                                           // OLED Display löschen (10 Punkt Schrift)
    String state = IoT_WLANstatus();                                // Verbindungsstatus
    if (IoT_WLANinited)                                             // Falls WLAN Verbindung erfolgreich, Status & IP im OLED Display anzeigen
    {
      IoT_DisplayDrawText(5, 0, "WLAN: " + IoT_WLANssid());         // WLAN Netzwerk Name
      if (state == "Connected")                                     // Wenn erfolgreich verbunden, dann Signalstärke hinzufügen
      {
        state = state + " (" + str(IoT_WLANrssi()) + "dBm)";        // Signalstärke
      }
      IoT_DisplayDrawText(5, 10, state);                            // Verbindungsstatus & Signalstärke
      IoT_DisplayDrawText(5, 20, "IP: " + IoT_WLANaddress(true));   // IP Adresse
      IoT_DisplayDrawText(5, 30, "GW: " + IoT_WLANgateway(true));   // IP Gateway
      IoT_DisplayDrawText(5, 40, "NT: " + IoT_WLANsubnet(true));    // IP Subnetzmaske
    }
    else                                                            // Falls WLAN Verbindung gescheitert ist, Status im OLED Display anzeigen
    {
      IoT_DisplayDrawText(5, 10, state); 
    }
  }
}

// *** NTP Funktionen ***

bool IoT_NTPinit ()                            // Funktion um die NTP Verbindung aufzubauen
{
  if (IoT_WLANinited)                             // Nur wenn WLAN verbunden ist
  {
    NTP.begin("pool.ntp.org", 1, true);           // Verbindung zu NTP Zeit-Server herstellen
    NTP.setInterval(63);                          // Aktualisierungsintervall setzen
    IoT_NTPinited = true;
    
    NTP.onNTPSyncEvent                            // Event-Handler starten
    (
      [ ] (NTPSyncEvent_t event)                  // Variable "event" enthält den aktuellen NTPSyncEvent_t
      {
        IoT_NTPcurrentEvent = event;              // "event" Variable global zur Verfügung stellen
        IoT_NTPeventAvail = true;                 // Anzeigen, dass ein Event empfangen wurde
      }
    );
    
  }
  else
  {
    IoT_NTPinited = false;                        // Ohne WLAN ist kein NTP möglich
  }
  IoT_Idle();
  return (IoT_NTPinited);
}

String IoT_NTPtime ()
{
  return (NTP.getTimeStr());                      // Uhrzeit im Format hh:mm:ss
}

String IoT_NTPdate ()
{
  String d = NTP.getDateStr();                    // Datum im Format tt/mm/jjjj
  return (replace(d, "/", "."));                  // Datum im Format tt.mm.jjjj           
}

String IoT_NTPdatetime ()
{
  String d = NTP.getTimeDateString();             // Datum & Uhrzeit im Format tt/mm/jjjj hh:mm:ss
  return (replace(d, "/", "."));                  // Datum & Uhrzeit im Format tt.mm.jjjj hh:mm:ss
}

bool IoT_NTPvalid ()
{
  return (IoT_NTPdate() != "01.01.1970");
}

String IoT_NTPdaylightSavingTime (bool germanLanguage)
{
  if (germanLanguage)
  {
    return (NTP.isSummerTime() ? "Sommerzeit" : "Winterzeit");
  }
  else
  {
    return (NTP.isSummerTime() ? "Summer Time" : "Winter Time");
  }
}

void IoT_NTPprintEvent (NTPSyncEvent_t event)
{
    if (event)                                                      // Fehlermeldung überprüfen
    {
        Serial.print("Time Sync error");
        if (event == noResponse)                                    // NTP-Server antwortet nicht
        {
            Serial.print(": NTP server not reachable");
        }
        else if (event == invalidAddress)                           // Fehlerhafte IP-Adresse für den NTP-Server
        {
            Serial.print(": Invalid NTP server address");
        }
        Serial.println("");
    }
    else                                                            // Zeit wurde erfolgreich empfangen
    {
        Serial.print("Got NTP time: ");
        Serial.println(NTP.getTimeDateString(NTP.getLastNTPSync()));
    }
}


// *** Utilites für den IoT-Brick ***

void IoT_Idle ()                                  // Verhindert das Festfrieren und Neustarten bei komplexen Aufgaben
{
  ESP.wdtFeed();                                  // Watchdog Timer zurücksetzen
//delay(1);                                       // Alternativ kann man auch 1 ms. warten, kostet aber CPU-Zeit
}

void IoT_WatchDog (bool active)                   // Schaltet den IoT-Brick-Watchdog ein (true) oder aus (false)
{
  ESP.wdtFeed();                                  // Watchdog Timer zurücksetzen
  if (active)
  {
    ESP.wdtEnable(5000);                          // Watchdog Timer einschalten auf 5 Sekunden (5000 ms.)
  }
  else
  {
    ESP.wdtDisable();                             // Watchdog Timer ausschalten
  }
  ESP.wdtFeed();                                  // Watchdog Timer zurücksetzen
}

bool IoT_Keypress ()                              // Eingebauten Taster abfragen
{
  return ((digitalRead(0) == LOW));
}

void IoT_WaitMessage ()
{
    IoT_DisplayClear(16);                         // OLED Display löschen (16 Punkt Schrift)
    IoT_DisplayAlignText(TEXT_ALIGN_CENTER);      // Zentrierte Darstellung des Textes
    IoT_DisplayDrawText(64,  5, "Den Taster am");
    IoT_DisplayDrawText(64, 25, "IoT-Brick drücken");
    IoT_DisplayDrawText(64, 45, "um fortzufahren.");
    IoT_DisplayUpdate();
    ESP.wdtFeed();                                // Watchdog Timer zurücksetzen
}

void IoT_WaitKeypress (uint64 timeout, bool message)
{
  long startTime = millis();
  if (timeout == 0)                               // Timeout 0 bedeutet: Kein Timeout
  {
    timeout = 20000000000000;                     // 20 Milliarden Sekunden = mehr als 630 Jahre
  }
  if (message)
  {
    IoT_WaitMessage ();
  }
  while (not(IoT_Keypress()) && (abs(millis() - startTime) < timeout))
  {                                               // Wenn Taster nicht gedrückt wird und noch kein Timeout
    delay(1);                                     // 1 ms. warten
    ESP.wdtFeed();                                // Watchdog Timer zurücksetzen
    IoT_WebServer.handleClient();                 // Bediene die http Anfragen
  }
  if (message)
  {
    IoT_DisplayClear(16);                         // OLED Display löschen (16 Punkt Schrift)
    IoT_DisplayUpdate();
  }
}

void IoT_WaitNoKeypress ()
{
  while (IoT_Keypress())                          // Warten, bis der Taster losgelassen wurde
  {
    delay(1);
    IoT_Idle();
    IoT_WebServer.handleClient();                 // Bediene die http Anfragen
  }
}

void IoT_TerminalWaitInit (bool showMessage)
{
  if (showMessage == t_ShowMessages)
  {
    if (IoT_OLEDinited)
    {
      IoT_DisplayClear(16);
      IoT_DisplayAlignText(TEXT_ALIGN_CENTER);
      IoT_DisplayDrawText(63, 2, "Bitte mit");
      IoT_DisplayDrawText(63, 22, "Terminal");
      IoT_DisplayDrawText(63, 42, "verbinden");
      IoT_DisplayUpdate();
    }
  }

  t_TerminalRespond = "";
  while (t_TerminalWidth == 0)
  {
    t_TerminalInit();
    if (t_TerminalWidth == 0)
    {
      delay(500); // 500 ms. warten
    }
  }
  t_TerminalRespond = "";

  if (showMessage == t_ShowMessages)
  {
    if (IoT_OLEDinited)
    {
      IoT_DisplayClear(10);
      IoT_DisplayAlignText(TEXT_ALIGN_CENTER);
      IoT_DisplayDrawText(63, 0, "Verbindung");
      IoT_DisplayDrawText(63, 12, "ist hergestellt");
      IoT_DisplayDrawText(63, 24, "Protokoll:");
      IoT_DisplayDrawText(63, 36, t_TerminalType);
      IoT_DisplayDrawText(63, 48, String(t_TerminalWidth) + " x " + String(t_TerminalHeight) + " Zeichen");
      IoT_DisplayUpdate();
    }
  }
}

void IoT_EEPROMclear ()                          // Die Konfiguration des WiFi-Managers wird hiervon nicht berührt
{
  bool success = true;
  const byte zero = 0;
  for (int i = 0; i <= IoT_EEPROMsize; i++)
  {
    EEPROM.put(i, zero);
  }
}

void IoT_EEPROMupdate ()
{
  EEPROM.commit();                               // Daten in das EEPROM zurückschreiben
}

void IoT_ShutDown ()
{
  if (IoT_OLEDinited)
  {
    IoT_Display.clear();                         // Display löschen
    IoT_Display.display();                       // Display aktualisieren
  }
  if (IoT_EEPROMsize > 0)
  {
    EEPROM.end();                                // Daten in das EEPROM zurückschreiben und Speicherplatz freigeben
  }
  while (true)
  {
    ESP.wdtFeed();                               // Watchdog Timer zurücksetzen
    t_TerminalGetKey();
    delay(100);                                  // 100 ms. warten
  }
}
